@@ -32,28 +32,6 @@ cgeneric_get <- function(model,
   stopifnot(!is.null(cgdata$ints))
   stopifnot(!is.null(cgdata$characters))
 
-  cmds <- c("graph", "Q", "initial", "mu", "log_prior")
-  cmd <- match.arg(cmd,
-                   cmds,
-                   several.ok = TRUE)
-  stopifnot(length(cmd)>0)
-
-  if(missing(theta)) {
-    if(cmd %in% c("Q", "log_prior")) {
-      stop("Please provide 'theta'!")
-    } else {
-      theta <- NULL
-      ntheta = 0L
-    }
-  } else {
-    if(inherits(theta, "matrix")) {
-      ntheta <- as.integer(ncol(theta))
-    } else {
-      ntheta <- 1L
-    }
-    theta <- as.numeric(theta)
-  }
-
   ## from Version 0.0.3.902 (and src/cgeneric_get.c)
   ## split each sparse matrix into list elements:
   ## nr, nc, m, i, j, x
@@ -73,6 +51,42 @@ cgeneric_get <- function(model,
     }
   }
 
+  cmds <- c("graph", "Q", "initial", "mu", "log_prior")
+  cmd <- match.arg(cmd,
+                   cmds,
+                   several.ok = TRUE)
+  stopifnot(length(cmd)>0)
+
+  if(missing(theta)) {
+    if(cmd %in% c("Q", "log_prior")) {
+      initheta <- .Call(
+        "inla_cgeneric_element_get",
+        "initial",
+        NULL,
+        as.integer(1),
+        cgdata$ints,
+        cgdata$doubles,
+        cgdata$characters,
+        cgdata$matrices,
+        cgdata$smatrices,
+        PACKAGE = "INLAtools"
+      )
+      if(length(initheta)>0) {
+        stop("Please provide 'theta'!")
+      }
+    } else {
+      theta <- NULL
+      ntheta = 0L
+    }
+  } else {
+    if(inherits(theta, "matrix")) {
+      ntheta <- as.integer(ncol(theta))
+    } else {
+      ntheta <- 1L
+    }
+    theta <- as.numeric(theta)
+  }
+
   if(length(cmd) == 1) {
     ret <- .Call(
       "inla_cgeneric_element_get",
@@ -141,12 +155,14 @@ cgeneric_get <- function(model,
   if(any(cmd == "Q")) {
     if(any(cmd == "graph")) {
       ij <- ret$graph
-      ret$graph <- Matrix::sparseMatrix(
-        i = ret$graph[[1]] + 1L,
-        j = ret$graph[[2]] + 1L,
-        x = rep(1, length(ret$graph[[1]])),
-        symmetric = TRUE,
-        repr = "T"
+      ret$graph <- Sparse(
+        Matrix::sparseMatrix(
+          i = ret$graph[[1]] + 1L,
+          j = ret$graph[[2]] + 1L,
+          x = rep(1, length(ret$graph[[1]])),
+          symmetric = TRUE,
+          repr = "T"
+        )
       )
       x <- ret$Q
       ret$Q <- ret$graph
@@ -164,12 +180,14 @@ cgeneric_get <- function(model,
         cgdata$smatrices,
         PACKAGE = "INLAtools"
       )
-      ret$Q <- Matrix::sparseMatrix(
-        i = ij[[1]] + 1L,
-        j = ij[[2]] + 1L,
-        x = ret$Q,
-        symmetric = TRUE,
-        repr = "T"
+      ret$Q <- Sparse(
+        Matrix::sparseMatrix(
+          i = ij[[1]] + 1L,
+          j = ij[[2]] + 1L,
+          x = ret$Q,
+          symmetric = TRUE,
+          repr = "T"
+        )
       )
     }
   }
@@ -206,6 +224,9 @@ graph.cgeneric <- function(model, optimize) {
 #' Retrieve the precision of an `cgeneric` object
 #' @export
 prec.cgeneric <- function(model, theta, optimize) {
+  if(missing(theta)) {
+    theta <- NULL
+  }
   if(missing(optimize)) {
     optimize <- FALSE
   }
